@@ -1,9 +1,7 @@
 import os
 import numpy as np
-from copy import copy
 
-from Orange.data import Table, StringVariable, Variable, DiscreteVariable, \
-    Domain
+from Orange.data import Table, StringVariable, Variable
 from Orange.data.io import FileFormat
 from Orange.widgets import gui
 from Orange.widgets.utils.itemmodels import VariableListModel, DomainModel
@@ -14,7 +12,6 @@ from Orange.widgets.widget import OWWidget, Msg, Input, Output
 from Orange.widgets.utils.concurrent import TaskState, ConcurrentWidgetMixin
 from orangecontrib.text.corpus import Corpus, get_sample_corpora_dir
 from orangecontrib.text.widgets.utils import widgets, QSize
-from orangecontrib.text.vectorization.base import get_unique_names
 
 
 class OWCorpus(OWWidget, ConcurrentWidgetMixin):
@@ -51,7 +48,6 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
     ])
     used_attrs = ContextSetting([])
     title_variable = ContextSetting("")
-    detect_languages = Setting(False)
 
     class Error(OWWidget.Error):
         read_file = Msg("Can't read file ({})")
@@ -86,7 +82,7 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
 
         # Used Text Features
         fbox = gui.widgetBox(self.controlArea, orientation=0)
-        ubox = gui.widgetBox(fbox, "Used text features", addSpace=False)
+        ubox = gui.widgetBox(fbox, "Used text features")
         self.used_attrs_model = VariableListModel(enable_dnd=True)
         self.used_attrs_view = VariablesListItemView()
         self.used_attrs_view.setModel(self.used_attrs_model)
@@ -98,16 +94,12 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
         aa.rowsRemoved.connect(self.update_feature_selection)
 
         # Ignored Text Features
-        ibox = gui.widgetBox(fbox, "Ignored text features", addSpace=False)
+        ibox = gui.widgetBox(fbox, "Ignored text features")
         self.unused_attrs_model = VariableListModel(enable_dnd=True)
         self.unused_attrs_view = VariablesListItemView()
         self.unused_attrs_view.setModel(self.unused_attrs_model)
         ibox.layout().addWidget(self.unused_attrs_view)
 
-        gui.checkBox(self.controlArea, self, "detect_languages",
-                     "Detect language automatically",
-                     callback=self.handle_languages)
-
         # Documentation Data Sets & Report
         box = gui.hBox(self.controlArea)
         self.browse_documentation = gui.button(
@@ -154,6 +146,7 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
     def open_file(self, path=None, data=None):
         self.closeContext()
         self.Error.clear()
+        self.cancel()
         self.unused_attrs_model[:] = []
         self.used_attrs_model[:] = []
         self.start(self._load_corpus, path, data)
@@ -166,7 +159,8 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
         self.update_output_info()
         self._setup_title_dropdown()
         self.used_attrs = list(self.corpus.text_features)
-        if not self.corpus.text_features:
+        all_str_features = [f for f in self.corpus.domain.metas if f.is_string]
+        if not all_str_features:
             self.Error.corpus_without_text_features()
             self.Outputs.corpus.send(None)
             return
@@ -254,7 +248,6 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
         if self.corpus is None:
             self.info.set_output_summary(self.info.NoOutput)
         else:
-            self.handle_languages()
             self.info.set_output_summary(
                 str(len(self.corpus)), describe(self.corpus))
 
@@ -312,65 +305,7 @@ class OWCorpus(OWWidget, ConcurrentWidgetMixin):
                 ('Target', describe(domain.class_vars)),
             ))
 
-    def handle_languages(self):
-        if self.corpus is not None:
-            domain = self.corpus.domain
-            if self.detect_languages:
-                if self.corpus.languages is None:
-                    self.corpus.detect_languages()
-
-                curr_attributes = list(domain.attributes)
-                curr_class_var = [domain.class_var] if domain.class_var else []
-                curr_metas = list(domain.metas)
-                curr_variables = curr_attributes + curr_class_var + curr_metas
-                curr_names = [var.name for var in curr_variables]
-                new_name = get_unique_names(curr_names, "Language")
-
-                variable_attrs = {'language-feature': True}
-                new_variable = StringVariable(new_name)
-                new_variable.attributes.update(variable_attrs)
-                new_domain = Domain(
-                    attributes=domain.attributes,
-                    class_vars=domain.class_var,
-                    metas=list(domain.metas) + [new_variable]
-                )
-                metas = np.hstack([self.corpus.metas,
-                                   np.array(self.corpus.languages).reshape(-1, 1)])
-                self.corpus = Corpus(new_domain,
-                                     self.corpus.X.copy(),
-                                     self.corpus.Y.copy(),
-                                     metas,
-                                     self.corpus.W.copy(),
-                                     copy(self.corpus.text_features))
-            else:
-                lang_feat_idx = None
-                for i, f in enumerate(domain.metas):
-                    if ('language-feature' in f.attributes and
-                       f.attributes['language-feature']):
-                        lang_feat_idx = i
-                        break
-                if lang_feat_idx is not None:
-                    new_domain = Domain(
-                        attributes=domain.attributes,
-                        class_vars=domain.class_var,
-                        metas=list(np.delete(list(domain.metas),
-                                             lang_feat_idx))
-                    )
-                    self.corpus = Corpus(
-                        new_domain,
-                        self.corpus.X.copy(),
-                        self.corpus.Y.copy(),
-                        np.delete(self.corpus.metas, lang_feat_idx, axis=1),
-                        self.corpus.W.copy(),
-                        copy(self.corpus.text_features)
-                    )
-        self.Outputs.corpus.send(self.corpus)
-
 
 if __name__ == '__main__':
-    from AnyQt.QtWidgets import QApplication
-    app = QApplication([])
-    widget = OWCorpus()
-    widget.show()
-    app.exec()
-    widget.saveSettings()
+    from orangewidget.utils.widgetpreview import WidgetPreview
+    WidgetPreview(OWCorpus).run()
